@@ -8,7 +8,7 @@ use CGI();
 BEGIN {
     use Exporter ();
     use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-    $VERSION     = '0.13';
+    $VERSION     = '0.14';
     @ISA         = qw(Exporter);
     #Give a hoot don't pollute, do not export more than needed by default
     @EXPORT      = qw();
@@ -22,6 +22,7 @@ BEGIN {
 our $DEFAULT_MODULE = 'InteractiveQuery2';
 
 our %DEFAULT_MODULES = (
+  'CustomOutput'      => 'CGI::OptimalQuery::CustomOutput',
   'PrinterFriendly'   => 'CGI::OptimalQuery::PrinterFriendly',
   'CSV'               => 'CGI::OptimalQuery::CSV',
   'InteractiveFilter' => 'CGI::OptimalQuery::InteractiveFilter',
@@ -40,7 +41,12 @@ our %DEFAULT_MODULES = (
 sub new {
   my $pack = shift;
   my $schema = $_[0];
-  $$schema{q} ||= new CGI();
+
+  if ($CGI::OptimalQuery::q) {
+    $$schema{q} = $CGI::OptimalQuery::q;
+  } else {
+    $$schema{q} ||= new CGI();
+  }
 
   # if this is a mod_perl query object, turn it into a CGI object
   if (! $$schema{q}->isa('CGI')) {
@@ -323,6 +329,9 @@ The following KEY/VALUES below describe OPTIONS used by the joins configuration.
 =item B<< new_cursor => 1 >>
 
 tells OptimalQuery to open a new cursor for this join. This can be used to select and filter multi-value fields.
+Optionally, an order_by param can be specified to sort the results returned by the cursor as such:
+
+=item B<< new_cursor_order_by => "some_field.id" >>
 
 =back
 
@@ -431,6 +440,9 @@ The default rows_page a user has when initially loading InteractiveQuery. Can al
 
 InteractiveQuery can optionally save searches to a database so users can revisit them latter. To do this, saved searches are tied to a unique user id. See the "Saved Searches" section for more information on this topic.
 
+=item B<< savedSearchAlerts => 0 | 1 >>
+
+If savedSearchUserID, is defined, setting savedSearchAlerts to 1 will enhance dialogs to optionally allow users to configure saved searches to alert them when records are added, removed, or are present. See the "Saved Search Alerts" section for additional details on configuring this feature.
 
 =item B<< sort => "[COLALIAS1] DESC, [COLALIAS2]" >>
 
@@ -600,6 +612,15 @@ Saved Searches are stored in a table in the database pointed to by the database
     oq_title VARCHAR(1000) NOT NULL,
     user_title VARCHAR(1000) NOT NULL,
     params TEXT,
+    -- 0:disabled, 1:added, 2:removed, 4:present
+    alert_mask INT UNSIGNED NOT NULL DEFAULT 0,
+    alert_interval_min INT UNSIGNED,
+    alert_dow VARCHAR(7),
+    alert_start_hour INT UNSIGNED,
+    alert_end_hour INT UNSIGNED,
+    alert_last_dt DATETIME,
+    alert_err TEXT,
+    alert_uids LONGTEXT,
     CONSTRAINT unq_oq_saved_search UNIQUE (user_id,uri,oq_title,user_title)
   );
 
@@ -611,6 +632,15 @@ Saved Searches are stored in a table in the database pointed to by the database
     oq_title VARCHAR2(1000) NOT NULL,
     user_title VARCHAR2(1000) NOT NULL,
     params CLOB,
+    -- 0:disabled, 1:added, 2:removed, 4:present
+    alert_mask NUMBER NOT NULL DEFAULT 0,
+    alert_interval_min NUMBER,
+    alert_dow VARCHAR2(7),
+    alert_start_hour NUMBER,
+    alert_end_hour NUMBER,
+    alert_last_dt DATE,
+    alert_err CLOB,
+    alert_uids CLOB,
     CONSTRAINT unq_oq_saved_search UNIQUE (user_id,uri,oq_title,user_title)
   );
   CREATE SEQUENCE s_oq_saved_search;
@@ -628,6 +658,68 @@ Optimal Query also provides a canned "Show My Saved Searches" HTML form componen
 Use CSS to stylize the output.
 
 
+=head1 SAVED SEARCH ALERTS
+
+Saved search alerts enhance Optimal Query to optionally allow users to configure saved searches to alert them when records are added, removed, or are present. To use this feature, follow the instructions in the "SAVED SEARCHES" section. You also need to make sure you set savedSearchAlerts to 1 in your schema if the current user is allowed to configure alerts.
+
+Lastly, you need to create a cron that will execute your OptimalQuery.
+
+=head1 SAVED SEARCH ALERTS CRON EXAMPLE
+
+
+    #!/usr/bin/perl
+
+    use strict;
+    use FindBin qw($Bin);
+    use lib "$Bin/../lib"; # include project lib
+    use DBI();
+    use CGI::OptimalQuery::SaveSearchTool();
+
+    # connect to your database
+    $APP::dbh ||= DBI->connect("dbi:SQLite:dbname=db/dat.db","","", { RaiseError => 1, PrintError => 1 });
+
+    CGI::OptimalQuery::SaveSearchTool::execute_saved_search_alerts(
+      # default is shown
+      # error_handler => sub { print STDERR @_; },
+
+      # if debug is true, no email is sent, emails will be logged to the error_handler
+      debug => 1,
+
+      # database handle
+      dbh => $APP::dbh,
+
+      base_url => "http://name.com/",
+
+      email_from => 'noreply@sr.unh.edu',
+
+      # define a handler which is called for each possible alert
+      # alerts aren't actually sent until the very end where they are batched
+      # and one email is sent for each email address containing one or more alerts
+      handler => sub {
+        # $o contains all the fields defined in the oq_saved_search rec
+        my ($o) = @_;
+
+        # you must set the email address for the search search owner id: $$o{USER_ID} 
+        $$o{email_to} = 'user_email@email.com';
+
+        # configure your application ENV
+        # local $APP::q = new CGI();
+        # local $APP::usr_id = $$o{USER_ID};
+        # local %APP::data = ();
+
+        # helper function to execute a perl module handler (will dynamically load module string and execute handler function)
+        #if ($$o{URI} =~ /(\w+)\.pm$/) {
+        #  CGI::OptimalQuery::SaveSearchTool::execute_handler("App::Applet::$1");
+        #}
+
+        # helper function to execute a perl script (will auto compile and cache a function)
+        if ($$o{URI} =~ /(\w+\.pl)$/) {
+          CGI::OptimalQuery::SaveSearchTool::execute_script("$Bin/cgi-bin/$1");
+        }
+      }
+    );
+
+
 =head1 FILTER GRAMMAR
 
   start: exp /^$/
